@@ -16,23 +16,31 @@
   synchronous operation on top of library for simple uses.
 - adapters.twisted_connection.TwistedConnection: Connection adapter for use
   with the Twisted framework
+- adapters.libev_connection.LibevConnection: Connection adapter for use
+  with the libev event loop and employing nonblocking IO
 
 """
-from base_connection import BaseConnection
-from asyncore_connection import AsyncoreConnection
-from blocking_connection import BlockingConnection
-from select_connection import SelectConnection
-from select_connection import IOLoop
+from pika.adapters.base_connection import BaseConnection
+from pika.adapters.asyncore_connection import AsyncoreConnection
+from pika.adapters.blocking_connection import BlockingConnection
+from pika.adapters.select_connection import SelectConnection
+from pika.adapters.select_connection import IOLoop
 
 # Dynamically handle 3rd party library dependencies for optional imports
 try:
-    from tornado_connection import TornadoConnection
+    from pika.adapters.tornado_connection import TornadoConnection
 except ImportError:
     TornadoConnection = None
 
 try:
-    from twisted_connection import TwistedConnection
-    from twisted_connection import TwistedProtocolConnection
+    from pika.adapters.twisted_connection import TwistedConnection
+    from pika.adapters.twisted_connection import TwistedProtocolConnection
 except ImportError:
     TwistedConnection = None
     TwistedProtocolConnection = None
+
+try:
+    from pika.adapters.libev_connection import LibevConnection
+except ImportError:
+    LibevConnection = None
+
