@@ -44,7 +44,8 @@ class SelectConnection(BaseConnection):
 
         """
         ioloop = IOLoop(self._manage_event_state)
-        super(SelectConnection, self).__init__(parameters, on_open_callback,
+        super(SelectConnection, self).__init__(parameters,
+                                               on_open_callback,
                                                on_open_error_callback,
                                                on_close_callback,
                                                ioloop,
@@ -57,12 +58,12 @@ class SelectConnection(BaseConnection):
         :rtype: bool
 
         """
-        if super(SelectConnection, self)._adapter_connect():
+        error = super(SelectConnection, self)._adapter_connect()
+        if not error:
             self.ioloop.start_poller(self._handle_events,
                                      self.event_state,
                                      self.socket.fileno())
-            return True
-        return False
+        return error
 
     def _flush_outbound(self):
         """Call the state manager who will figure out that we need to write then
@@ -131,6 +132,7 @@ class IOLoop(object):
     def start(self):
         """Start the IOLoop, waiting for a Poller to take over."""
         LOGGER.debug('Starting IOLoop')
+        self.poller.open = True
         while not self.poller:
             time.sleep(SelectPoller.TIMEOUT)
         self.poller.start()
@@ -146,12 +148,7 @@ class IOLoop(object):
         """
         LOGGER.debug('Starting the Poller')
         self.poller = None
-        if hasattr(select, 'poll') and hasattr(select.poll, 'modify'):
-            if not SELECT_TYPE or SELECT_TYPE == 'poll':
-                LOGGER.debug('Using PollPoller')
-                self.poller = PollPoller(fileno, handler, events,
-                                         self._manage_event_state)
-        if not self.poller and hasattr(select, 'epoll'):
+        if hasattr(select, 'epoll'):
             if not SELECT_TYPE or SELECT_TYPE == 'epoll':
                 LOGGER.debug('Using EPollPoller')
                 self.poller = EPollPoller(fileno, handler, events,
@@ -161,6 +158,11 @@ class IOLoop(object):
                 LOGGER.debug('Using KQueuePoller')
                 self.poller = KQueuePoller(fileno, handler, events,
                                            self._manage_event_state)
+        if not self.poller and hasattr(select, 'poll') and hasattr(select.poll(), 'modify'):
+            if not SELECT_TYPE or SELECT_TYPE == 'poll':
+                LOGGER.debug('Using PollPoller')
+                self.poller = PollPoller(fileno, handler, events,
+                                         self._manage_event_state)
         if not self.poller:
             LOGGER.debug('Using SelectPoller')
             self.poller = SelectPoller(fileno, handler, events,
@@ -202,7 +204,7 @@ class SelectPoller(object):
         self.events = events
         self.open = True
         self._handler = handler
-        self._timeouts = dict()
+        self._timeouts = []
         self._manage_event_state = state_manager
 
     def add_timeout(self, deadline, callback_method):
@@ -219,7 +221,7 @@ class SelectPoller(object):
         value = {'deadline': time.time() + deadline,
                  'callback': callback_method}
         timeout_id = hash(frozenset(value.items()))
-        self._timeouts[timeout_id] = value
+        self._timeouts.append((timeout_id, value))
         return timeout_id
 
     def flush_pending_timeouts(self):
@@ -252,7 +254,7 @@ class SelectPoller(object):
                                                output_fileno,
                                                error_fileno,
                                                SelectPoller.TIMEOUT)
-        except select.error, error:
+        except select.error as error:
             return self._handler(self.fileno, ERROR, error)
 
         # Build our events bit mask
@@ -270,13 +272,17 @@ class SelectPoller(object):
     def process_timeouts(self):
         """Process the self._timeouts event stack"""
         start_time = time.time()
-        for timeout_id in self._timeouts.keys():
-            if timeout_id not in self._timeouts:
-                continue
-            if self._timeouts[timeout_id]['deadline'] <= start_time:
-                callback = self._timeouts[timeout_id]['callback']
-                del self._timeouts[timeout_id]
+        # while loop instead of a more straightforward for loop so we can
+        # delete items from the list while iterating
+        i = 0
+        while i < len(self._timeouts):
+            t_id, timeout = self._timeouts[i]
+            if timeout['deadline'] <= start_time:
+                callback = timeout['callback']
+                del self._timeouts[i]
                 callback()
+            else:
+                i += 1
 
     def remove_timeout(self, timeout_id):
         """Remove a timeout if it's still in the timeout stack
@@ -284,8 +290,11 @@ class SelectPoller(object):
         :param str timeout_id: The timeout id to remove
 
         """
-        if timeout_id in self._timeouts:
-            del self._timeouts[timeout_id]
+        for i in xrange(len(self._timeouts)):
+            t_id, timeout = self._timeouts[i]
+            if t_id == timeout_id:
+                del self._timeouts[i]
+                break
 
     def start(self):
         """Start the main poller loop. It will loop here until self.closed"""
@@ -375,7 +384,7 @@ class KQueuePoller(SelectPoller):
         events = 0
         try:
             kevents = self._kqueue.control(None, 1000, SelectPoller.TIMEOUT)
-        except OSError, error:
+        except OSError as error:
             return self._handler(self.fileno, ERROR, error)
         for event in kevents:
             if event.filter == select.KQ_FILTER_READ and READ & self.events:
@@ -430,7 +439,7 @@ class PollPoller(SelectPoller):
             LOGGER.info("Unregistering poller on fd %d" % self.fileno)
             self.update_handler(self.fileno, 0)
             self._poll.unregister(self.fileno)
-        except IOError, err:
+        except IOError as err:
             LOGGER.debug("Got IOError while shutting down poller: %s", err)
 
     def poll(self, write_only=False):
@@ -439,7 +448,10 @@ class PollPoller(SelectPoller):
         :param bool write_only: Only process write events
 
         """
-        events = self._poll.poll(int(SelectPoller.TIMEOUT * 1000))
+        try:
+            events = self._poll.poll(int(SelectPoller.TIMEOUT * 1000))
+        except select.error as error:
+            return self._handler(self.fileno, ERROR, error)
         if events:
             LOGGER.debug("Calling %s with %d events",
                          self._handler, len(events))
@@ -472,7 +484,10 @@ class EPollPoller(PollPoller):
         :param bool write_only: Only process write events
 
         """
-        events = self._poll.poll(SelectPoller.TIMEOUT)
+        try:
+            events = self._poll.poll(SelectPoller.TIMEOUT)
+        except IOError as error:
+            return self._handler(self.fileno, ERROR, error)
         if events:
             LOGGER.debug("Calling %s", self._handler)
             for fileno, event in events:
